import tomllib

from flask import Flask
from whitenoise import WhiteNoise


# noinspection PyUnresolvedReferences
def create_app():
    # create and configure the app
    app = Flask(__name__)

    app.config.from_file("config.toml", load=tomllib.load, text=False)
    app.config["MAX_CONTENT_LENGTH"] *= 1024 * 1024 * 1024  # bytes -> megabytes -> gigabytes

    # static file handling
    app.wsgi_app = WhiteNoise(app.wsgi_app, root="static/", prefix="static/", max_age=172800)  # 2 days

    with open("localisations.toml", "rb") as file:
        localisations = tomllib.load(file)

    for language in app.config["ENABLED_LOCALISATIONS"]:
        check_localisation(app.logger, localisations, language)

    app.config["LOCALISATIONS"] = localisations

    app.logger.info("Localisations loaded.")

    with app.app_context():
        import routes

    return app


def check_localisation(logger, localisations: dict, language: str, upper_name: str = ""):
    if len(localisations) == 0:
        logger.warning("Possibly missing '{}' localisation for '{}'".format(language, upper_name[1:]))
        return

    for name, localisation in localisations.items():
        try:
            _ = localisation[language]
        except KeyError:
            try:
                check_localisation(logger, localisation, language, "{}.{}".format(upper_name, name))
            except KeyError as e:
                raise e
        except TypeError:
            logger.error("Missing '{}' localisation for '{}'.".format(language, upper_name[1:]))
