# Architecture overview

1. Webserver takes and validates user submitted files
2. Cron job scans files and enqueues new jobs on cluster
3. Job gets processed on the cluster using [WhisperX](https://github.com/m-bain/whisperX)
4. Mailservice scans job folders for completed jobs and:
   - Sends processed files to users
   - Optional: Notifies admins on processing errors
   - Optional: Sends monitoring data to webserver 


## Job folders
Jobs are given a randomly generated uuid. A job folder looks like this:

```
job_uuid:
  - audio.mkv
  - video_language.txt
  - metadata.json
  - statefile (new/new_transcribe/done/error)
```

### audio.mkv
Preprocessed input file. Contains only audio data to conserve disk space.

### video_language.txt
Contains the video language tag used for processing with [WhisperX](https://github.com/m-bain/whisperX).
Is used by the cronjob script (step 3).

### metadata.json
Used by mailservice (step 4).
```json
{
   "job_type": "subtitle",
   "email": "example@example.com",
   "language": "de",
   "video_language": "de",
   "filename": "original_filename.original_file_extension"
}
```

### statefile
State is tracked through the following files in the jobs folder:
   - new: Subtitle job has been submitted by user
   - new_transcribe: Transcript job has been submitted by user
   - submitted: Job has been scheduled on gpu cluster
   - done: Job has been processed without errors
   - error: Job has been processed with errors