import av
from flask import current_app
from flask_wtf import FlaskForm
from flask_wtf.file import FileField, FileRequired, FileSize
from wtforms import StringField, SelectField, IntegerField
from wtforms.validators import InputRequired, Email, Optional
from wtforms.validators import ValidationError


def validate_audio(_, field):
    file = field.data
    try:
        with av.open(file) as container:
            has_audio = len([stream for stream in container.streams if stream.type == "audio"]) > 0

        # reset file stream
        file.seek(0)

        if not has_audio:
            raise ValidationError("noaudiotrack")
    except av.FFmpegError as e:
        current_app.logger.info(
            "Error while checking audio of file '{}': {}".format(file.filename, str(e.strerror))
        )
        raise ValidationError("brokenfile")


class UploadForm(FlaskForm):
    email = StringField(
        current_app.config["LOCALISATIONS"]["email"]["label"][current_app.config["DEFAULT_LANGUAGE"]],
        validators=[InputRequired("invalidEmail"), Email("invalidEmail")],
    )
    language = SelectField(
        current_app.config["LOCALISATIONS"]["language"]["label"][current_app.config["DEFAULT_LANGUAGE"]],
        validators=[InputRequired("required")],
    )
    file = FileField(
        current_app.config["LOCALISATIONS"]["file"]["label"][current_app.config["DEFAULT_LANGUAGE"]],
        validators=[
            FileRequired("nofile"),
            FileSize(current_app.config["MAX_CONTENT_LENGTH"]),
            validate_audio,
        ],
    )